uniform sampler2D ShadowTexture;
uniform sampler2D Texture0;
uniform int BoolShadow;

varying vec4 ShadowKoordinaten;
varying vec4 ambientGlobal;
varying vec3 normal;
varying vec3 v;
varying vec3 l;
varying float distance;

vec4 ShadowKoordinatenW;

float chebyshevUpperBound( float distance)
{
	float variance;
	float d;
	float p_max;

	vec2 moments = texture2D(ShadowTexture,ShadowKoordinatenW.xy).rb;
	
	if (distance <= moments.x)	return 1.0 ;

	variance = moments.y - (moments.x*moments.x);
	variance = max(variance,0.00002);

	d = distance - moments.x;
	p_max = variance / (variance + d*d);

	return p_max;
}


void main()
{	
	if(BoolShadow == 1)	
	{
		vec3 Eye             = normalize(-v);
		vec3 Reflected       = normalize( reflect( -l, normal ));
		ShadowKoordinatenW	 = ShadowKoordinaten / ShadowKoordinaten.w;
		float shadow = chebyshevUpperBound(ShadowKoordinatenW.z);

		float att			= 1.0 / (gl_LightSource[0].constantAttenuation + gl_LightSource[0].linearAttenuation * distance + gl_LightSource[0].quadraticAttenuation * pow(distance,2.0));
	
		vec4 IAmbient       = gl_LightSource[0].ambient * gl_FrontMaterial.ambient;
		vec4 IDiffuse       = gl_LightSource[0].diffuse * max(dot(normal, l), 0.0)* gl_FrontMaterial.diffuse;
		vec4 ISpecular      = gl_LightSource[0].specular * pow(max(dot(Reflected, Eye), 0.0), gl_FrontMaterial.shininess) * gl_FrontMaterial.specular;	
		gl_FragColor		= (ambientGlobal* texture2D(Texture0, vec2(gl_TexCoord[0]))) + (att * (IAmbient* texture2D(Texture0, vec2(gl_TexCoord[0])) )) + (att * (vec4(shadow) * ((texture2D(Texture0, vec2(gl_TexCoord[0])) * IDiffuse) + ISpecular)));
	}
	else
	{
		if(BoolShadow == 2)
		{
			gl_FragColor = texture2D(Texture0, vec2(gl_TexCoord[0]))*gl_Color;
		}
		else
		{
			vec3 Eye             = normalize(-v);
			vec3 Reflected       = normalize( reflect( -l, normal )); 
			float att = 1.0 / (gl_LightSource[0].constantAttenuation + gl_LightSource[0].linearAttenuation * distance + gl_LightSource[0].quadraticAttenuation * pow(distance,2.0));

			vec4 IAmbient        = gl_LightSource[0].ambient  * gl_FrontMaterial.ambient;
			vec4 IDiffuse        = gl_LightSource[0].diffuse  * max(dot(normal, l), 0.0)								   * gl_FrontMaterial.diffuse;
			vec4 ISpecular       = gl_LightSource[0].specular * pow(max(dot(Reflected, Eye), 0.0), gl_FrontMaterial.shininess) * gl_FrontMaterial.specular;
		
			gl_FragColor = (ambientGlobal* texture2D(Texture0, vec2(gl_TexCoord[0]))) + (att * (IAmbient* texture2D(Texture0, vec2(gl_TexCoord[0])) )) + (att * ((texture2D(Texture0, vec2(gl_TexCoord[0])) * IDiffuse) + ISpecular));
		}
	}
}
